##########################################################################
#                                                                        #
# A company wants to maximize the combined Net Present Value (NPV) of a  #
# maximum of 6 opportunities that require 6 yearly investments. In each  #
# year there is only a limited amount of money available. All amounts    #
# are given in millions of dollars.                                      # 
#                                                                        #
##########################################################################

set OPPORTUNITIES;
set YEARS ordered;

##############
# Parameters #
##############

param EXPECTED_CASH_FLOW {YEARS,OPPORTUNITIES};   # Expected cash flow
param BUDGET {YEARS};                             # Available budget
param INTEREST;                                   # Interest rate

#############
# Variables #
#############

var INVESTMENTS {OPPORTUNITIES} binary;           # Percentage to invest
var TOTAL_CASH_FLOW {YEARS};                      # Resulting cash flow 
var NPV {OPPORTUNITIES};                          # Net Present Value

#########
# Model #
#########

maximize TOTAL_NPV:
   sum {i in OPPORTUNITIES} INVESTMENTS[i] * NPV[i];
subject to
   Calculate_NPV {i in OPPORTUNITIES}:
      NPV[i] = sum {j in YEARS} EXPECTED_CASH_FLOW[j,i]/((1+INTEREST)^ord(j));      
   Calculate_TOTAL_CASH_FLOW {j in YEARS}:   
      TOTAL_CASH_FLOW[j] = sum {i in OPPORTUNITIES} INVESTMENTS[i]*
                                EXPECTED_CASH_FLOW[j,i];
   Budget_must_suffice {j in YEARS}:
      TOTAL_CASH_FLOW[j] + BUDGET[j] >= 0;

