###########################################################################
#                                                                         #
# What is the best ordering policy for a warehouse to minimize cost       #
# while meeting demand? The warehouse has a limited storage capacity.     #
#                                                                         #
###########################################################################

set PRODUCTS;

##############
# Parameters #
##############

param ORDERING_COST {PRODUCTS};               # Cost to order
param HOLDING_COST {PRODUCTS};                # Cost to store
param STORAGE_PER_UNIT {PRODUCTS};            # Space occupied per unit
param MONTHLY_DEMAND {PRODUCTS};              # Monthly demand per product
param TOTAL_STORAGE;                          # Total space available

#############
# Variables #
#############

var QUANTITY {PRODUCTS} >=0.01;              # Number to order

#########
# Model #
#########

minimize TOTAL_COST:
    sum {i in PRODUCTS} (HOLDING_COST[i]*QUANTITY[i]/2+
                         ORDERING_COST[i]*MONTHLY_DEMAND[i]*(QUANTITY[i])^-1);
subject to
   Available_space:
      sum {i in PRODUCTS} QUANTITY[i]*STORAGE_PER_UNIT[i]/2 <= TOTAL_STORAGE;

