###########################################################################
#                                                                         #
# A company wants to minimize the costs of shipping goods from production #
# plants to warehouses near metropolitan demand centers, while not        #
# exceeding the supply available from each plant and meeting the demand   #
# from each metropolitan area. The company currently has three plants and #
# it is thinking about opening another in Arkansas.                       #
#                                                                         #
###########################################################################

set PLANTS ordered;
set WAREHOUSES;

##############
# Parameters #
##############

param DEMAND {WAREHOUSES};               # Demand of warehouses
param SUPPLY {PLANTS};                   # Maximum supply at plants
param SHIPPING_COST {PLANTS,WAREHOUSES}; # Cost of shipping
param EXTRA_COST;                        # Extra cost if new plant is opened

#############
# Variables #
#############

var NUMBER_TO_SHIP {PLANTS,WAREHOUSES} >=0;  # Products to ship
var DECISION binary;                         # Decision to open new plant

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in PLANTS, j in WAREHOUSES} NUMBER_TO_SHIP[i,j]*SHIPPING_COST[i,j]+
        EXTRA_COST*DECISION;
subject to
   Available_supply {i in PLANTS}:
      sum {j in WAREHOUSES} NUMBER_TO_SHIP[i,j] <= if last(PLANTS)=i
          then SUPPLY[i]*DECISION
          else SUPPLY[i];
   The_demand {j in WAREHOUSES}:
      sum {i in PLANTS} NUMBER_TO_SHIP[i,j] = DEMAND[j];
