##########################################################################
#                                                                        #
# An investor wants to put together a portfolio consisting of a          #
# maximum of 6 bonds. What is the best combination of bonds to get the   #
# optimum yield within a given investment time horizon?                  #
#                                                                        #
##########################################################################

set BONDS;               

##############
# Parameters #
##############

param DURATION {BONDS} >0;              # Duration of each bond in years
param YIELD {BONDS};                    # Yield to maturity for each bond
param INVESTMENT_TIME_HORIZON;          # The investment time horizon

#############
# Variables #
#############

var PORTFOLIO_PART {BONDS} >=0, <=1;    # Percentage of portfolio per bond

#########
# Model #
#########

maximize PORTFOLIO_YIELD: 
   sum {i in BONDS} PORTFOLIO_PART[i]*YIELD[i];
subject to 
   PORTFOLIO_DURATION:
      sum {i in BONDS} PORTFOLIO_PART[i]*DURATION[i]=INVESTMENT_TIME_HORIZON;
   COMPLETE_PORTFOLIO:
      sum {i in BONDS} PORTFOLIO_PART[i] = 1;

