#########################################################################
#                                                                       #
# What is the minimum cost portfolio, consisting of several bonds, that #
# provides enough cash flow to cover liabilities in each period?        #
#                                                                       #
#########################################################################

set BONDS;
set YEARS ordered;

##############
# Parameters #
##############

param FACE_VALUE {BONDS} >0;               # Face value of the bonds
param PRICE {BONDS} >0;                    # Prices of the bonds
param YEARS_TO_MATURITY {BONDS} >0;        # Years to maturity for each bond
param COUPON_PAYMENTS {BONDS} >0;          # The annual coupon payment 
param LIABILITIES {YEARS};                 # Liabilities each year
param INTEREST;                            # Interest rate

#############
# Variables #
#############

var NUMBER_PURCHASED {BONDS} integer, >=0; # Numer of bonds to purchase
var CASH_FLOW {YEARS};                     # Cash flow in each year

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in BONDS} NUMBER_PURCHASED[i] * PRICE[i];
subject to
   Initial_Year {j in YEARS: ord(j) = 1}:
   CASH_FLOW [j] = sum {i in BONDS: YEARS_TO_MATURITY[i] >= ord(j)} 
                       NUMBER_PURCHASED[i] * COUPON_PAYMENTS[i];
   The_Cash_Flow {j in YEARS: ord(j) >1} :
   CASH_FLOW[j] = (CASH_FLOW[prev(j)] - LIABILITIES[prev(j)])*(1+INTEREST)+
                  sum {i in BONDS: YEARS_TO_MATURITY[i] >= ord(j)} 
                       NUMBER_PURCHASED[i] * COUPON_PAYMENTS[i];
   The_Liabilities { j in YEARS} :
   CASH_FLOW[j] >= LIABILITIES[j];

