##########################################################################
#                                                                        #
# A fuel truck with several compartments needs to supply different types #
# of gas to a customer. When demand is not filled, the company loses a   #
# certain amount for each gallon that is not delivered. How should the   #
# truck be loaded so as to minimize loss?                                #
#                                                                        #
##########################################################################

set GASES;
set COMPARTMENTS;

##############
# Parameters #
##############

param LOSS_PER_GALLON;                   # Loss per gallon not delivered
param COMPARTMENT_SIZE {COMPARTMENTS};   # Size of different compartments
param DEMAND {GASES};                    # Demands of different types of gas

#############
# Variables #
#############

var DECISIONS {GASES, COMPARTMENTS} binary; # Decisions to fill compartment
                                            # with gas
var AMOUNTS {GASES, COMPARTMENTS} >=0;      # Amount of gas in compartment

#########
# Model #
#########

minimize TOTAL_LOSS:
   sum {i in GASES} (DEMAND[i]-sum {j in COMPARTMENTS} AMOUNTS[i,j])*
       LOSS_PER_GALLON;
subject to
   Only_one_gas_per_compartment {j in COMPARTMENTS}:
      sum {i in GASES} DECISIONS[i,j] = 1; 
   Compartment_limits { i in GASES, j in COMPARTMENTS}:   
      AMOUNTS[i,j] <= COMPARTMENT_SIZE[j]*DECISIONS[i,j];
   Demand {i in GASES}:
      sum {j in COMPARTMENTS} AMOUNTS[i,j] <=DEMAND[i];
