###########################################################################
#                                                                         #
# Minimize the costs of shipping goods from factories to warehouses and   #
# customers and warehouses to customers, while not exceeding the supply   #
# available from each factory or the capacity of each warehouse, and      #
# meeting the demand from each customer.                                  #
#                                                                         #
###########################################################################


set FACTORIES;
set CUSTOMERS;
set WAREHOUSES;

##############
# Parameters #
##############

param FACTORY_CAPACITY {FACTORIES};         # Capacity of factories
param WAREHOUSE_CAPACITY {WAREHOUSES};      # Capacity of warehouses
param DEMAND {CUSTOMERS};                   # Demand of customers
param FAC_CUS_COST {FACTORIES,CUSTOMERS};   # Shipping cost
param FAC_WARE_COST {FACTORIES,WAREHOUSES}; #     ,,
param WARE_CUS_COST {WAREHOUSES,CUSTOMERS}; #     ,,

#############
# Variables #
#############

var FAC_CUS_SHIP {FACTORIES,CUSTOMERS} >=0;        # Products shippped
var FAC_WARE_SHIP {FACTORIES,WAREHOUSES} >=0;      #    ,,
var WARE_CUS_SHIP {WAREHOUSES,CUSTOMERS} >=0;      #    ,,

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in FACTORIES, j in CUSTOMERS} 
       FAC_CUS_COST[i,j]*FAC_CUS_SHIP[i,j]+
   sum {i in FACTORIES, k in WAREHOUSES}    
       FAC_WARE_COST[i,k]*FAC_WARE_SHIP[i,k]+
   sum {k in WAREHOUSES, j in CUSTOMERS}    
       WARE_CUS_COST[k,j]*WARE_CUS_SHIP[k,j];
subject to
  Demand {j in CUSTOMERS}:
     sum {i in FACTORIES} FAC_CUS_SHIP[i,j]+ 
     sum {k in WAREHOUSES} WARE_CUS_SHIP[k,j] = DEMAND[j];
  Factory_capacity {i in FACTORIES}:
     sum {j in CUSTOMERS} FAC_CUS_SHIP[i,j]+ 
     sum {k in WAREHOUSES} FAC_WARE_SHIP[i,k] <= FACTORY_CAPACITY[i];
  Warehouse_capacity {k in WAREHOUSES}:
     sum {i in FACTORIES} FAC_WARE_SHIP[i,k] <= WAREHOUSE_CAPACITY[k];
  Flow {k in WAREHOUSES}:
     sum {i in FACTORIES} FAC_WARE_SHIP[i,k] = 
     sum {j in CUSTOMERS} WARE_CUS_SHIP[k,j];


    

