############################################################################
#                                                                          #
# Minimize the cost of operating different types of machines while meeting #
# the demand. Each machine has a different cost and capacity. There are    #
# only a limited number of machines available of each type.                #
#                                                                          #
############################################################################

set MACHINES;
set WEEKDAYS ordered;

##############
# Parameters #
##############

param INITIAL_COST {MACHINES};               # Cost to start up machines
param ADDITIONAL_COST {MACHINES};            # Extra cost per product
param MAXIMUM_PRODUCTS {MACHINES};           # Maximum products per day
param AVAILABLE_MACHINES {MACHINES};         # Available machines
param DEMAND {WEEKDAYS};                     # Required number of products

#############
# Variables #
#############

var MACHINES_TO_USE {MACHINES,WEEKDAYS} integer; # Number of machines to use
var PRODUCTS_TO_MAKE {MACHINES,WEEKDAYS};        # Number of products to make
var CARRY_OVER {WEEKDAYS};                       # Extra products for next day

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in MACHINES, j in WEEKDAYS} (MACHINES_TO_USE[i,j]*INITIAL_COST[i]+
      PRODUCTS_TO_MAKE[i,j]*ADDITIONAL_COST[i]);
subject to
   Available_machines {i in MACHINES, j in WEEKDAYS}:
      MACHINES_TO_USE[i,j] <= AVAILABLE_MACHINES[i];
   Capacity {i in MACHINES, j in WEEKDAYS}:
      PRODUCTS_TO_MAKE[i,j] <= MACHINES_TO_USE[i,j]*MAXIMUM_PRODUCTS[i];
   Demand {j in WEEKDAYS}:
      sum {i in MACHINES} PRODUCTS_TO_MAKE[i,j] +
             if j=first(WEEKDAYS) then 0 else CARRY_OVER[prev(j)]          
             >= DEMAND[j];
   Planning {j in WEEKDAYS}:
     CARRY_OVER[j] = if j=first(WEEKDAYS)
        then sum {i in MACHINES} PRODUCTS_TO_MAKE[i,j]-DEMAND[j]
        else sum {i in MACHINES} PRODUCTS_TO_MAKE[i,j]+
             CARRY_OVER[prev(j)]-DEMAND[j];


