##########################################################################
#                                                                        #
# What chemical plants should be used and how much should they produce   #
# to meet a certain quality (amounts of copper and silver) and minimize  #
# cost? There are several plants with different quality, capacity and    #
# cost to operate. A minimum output per year is required.                #
#                                                                        #
##########################################################################

set PLANTS;
set YEARS;

##############
# Parameters #
##############

param RELATIVE_COPPER_CONTENTS {PLANTS};        # Compared to required
param RELATIVE_SILVER_CONTENTS {PLANTS};        #        ,,
param MAXIMUM_PRODUCTION {PLANTS};              # Maximum output per year
param OPERATING_COST {PLANTS};                  # Cost to operate plant
param REQUIRED {YEARS};                         # Total required output
param COPPER_REQUIRED {YEARS};                  # Required copper contents
param SILVER_REQUIRED {YEARS};                  # Required silver contents

#############
# Variables #
#############

var AMOUNTS_TO_PRODUCE {PLANTS,YEARS} >=0;      # Amount per plant
var DECISIONS {PLANTS,YEARS} binary;            # Decision to open plant

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in PLANTS, j in YEARS} DECISIONS[i,j]*OPERATING_COST[i]; 
subject to 
   Max_3_plants_a_year {j in YEARS}:
      sum {i in PLANTS} DECISIONS[i,j] <= 3;
   Output {j in YEARS}:
      sum {i in PLANTS} AMOUNTS_TO_PRODUCE[i,j] >= REQUIRED[j];
   Maximum {i in PLANTS,j in YEARS}:
      AMOUNTS_TO_PRODUCE[i,j] <= MAXIMUM_PRODUCTION[i]*DECISIONS[i,j];
   Copper {j in YEARS}:
      sum {i in PLANTS} AMOUNTS_TO_PRODUCE[i,j]*RELATIVE_COPPER_CONTENTS[i]
                        >= REQUIRED[j]*COPPER_REQUIRED[j];
   Silver {j in YEARS}:
      sum {i in PLANTS} AMOUNTS_TO_PRODUCE[i,j]*RELATIVE_SILVER_CONTENTS[i]
                        >= REQUIRED[j]*SILVER_REQUIRED[j];

