###########################################################################
#                                                                         #
# A planing mill uses several different types of planers. What planers    #
# should the company use to minimize cost? Limited time is available.     #
#                                                                         #
###########################################################################

set PLANERS;
set WOODSIZES;

##############
# Parameters #
##############

param SPEED {PLANERS};                  # Speed in feet per minute
param COST {PLANERS};                   # Cost to operate planer per hour
param MAXIMUM_SIZE {PLANERS};           # Maximum size planer can handle
param DEMAND {WOODSIZES};               # Amount of wood that needs planing
param AVAILABLE_TIME;                   # Time allowed for job

set POSSIBLE := {i in PLANERS,j in WOODSIZES: j<= MAXIMUM_SIZE[i]};

#############
# Variables #
#############

var WOOD {(i,j) in POSSIBLE} >=0; # Wood to plane

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in PLANERS,(i,j) in POSSIBLE} (COST[i]/(SPEED[i]*60)*WOOD[i,j]);
subject to
   Time_limit {i in PLANERS}:
   sum {(i,j) in POSSIBLE} WOOD[i,j] <= AVAILABLE_TIME*SPEED[i]*60;
   Demand {j in WOODSIZES}:
      sum {(i,j) in POSSIBLE} WOOD[i,j] = DEMAND[j];




