#############################################################################
#                                                                           #
# A small airline company maintains 2 daily flights between Salt Lake City, #
# Chicago and Dallas. How should the company schedule the crews to          #
# minimize cost?                                                            #
#                                                                           #
# FLIGHT SCHEDULE                                                           #
#    FROM             TO          DEPARTURE  ARRIVAL   DEPARTURE  ARRIVAL   #
#                                                                           #
# Salt Lake City     Dallas        9:00 AM  12:00 PM    2:00 PM   5:00 PM   #
# Salt Lake City    Chicago       10:00 AM   2:00 PM    3:00 PM   7:00 PM   #
#     Dallas     Salt Lake City    8:00 AM  11:00 AM    2:00 PM   5:00 PM   #
#     Dallas        Chicago        9:00 AM  11:00 AM    3:00 PM   5:00 PM   #
#    Chicago     Salt Lake City    8:00 AM  12:00 PM    2:00 PM   6:00 PM   #
#    Chicago         Dallas       10:00 AM  12:00 PM    4:00 PM   6:00 PM   #
#                                                                           #
# A crew must land and arrive in the same city. It is possible to fly the   #
# crew back aboard another airline. This would always be on a 8:00 PM       # 
# flight. There are 6 airplanes in use.                                     #
#                                                                           #  
#############################################################################

set ROTATIONS;
set FLIGHT1 within ROTATIONS;
set FLIGHT2 within ROTATIONS;
set FLIGHT3 within ROTATIONS;
set FLIGHT4 within ROTATIONS;
set FLIGHT5 within ROTATIONS;
set FLIGHT6 within ROTATIONS;
set FLIGHT7 within ROTATIONS;
set FLIGHT8 within ROTATIONS;
set FLIGHT9 within ROTATIONS;
set FLIGHT10 within ROTATIONS;
set FLIGHT11 within ROTATIONS;
set FLIGHT12 within ROTATIONS;

##############
# Parameters #
##############

param FLYING_HOURS {ROTATIONS};               # Flight hours per rotation
param OTHER_HOURS {ROTATIONS};                # Other hours per rotation
param COST_FLYING_HOURS;                      # Crew cost per flying hour
param COST_OTHER_HOURS;                       # Crew cost per other hour

#############
# Variables #
#############

var DECISIONS {ROTATIONS} binary;             # Rotation decisions                

#########
# Model #
#########

minimize TOTAL_COST:
   sum {i in ROTATIONS} DECISIONS[i]*(FLYING_HOURS[i]*COST_FLYING_HOURS+
                                     OTHER_HOURS[i]*COST_OTHER_HOURS);
subject to
   # All 12 flights must have exactly one crew.
   Flight1:
     sum {i in FLIGHT1} DECISIONS[i] = 1;
   Flight2:
     sum {i in FLIGHT2} DECISIONS[i] = 1;
   Flight3:
     sum {i in FLIGHT3} DECISIONS[i] = 1;
   Flight4:
     sum {i in FLIGHT4} DECISIONS[i] = 1;
   Flight5:
     sum {i in FLIGHT5} DECISIONS[i] = 1;
   Flight6:
     sum {i in FLIGHT6} DECISIONS[i] = 1;
   Flight7:
     sum {i in FLIGHT7} DECISIONS[i] = 1;
   Flight8:
     sum {i in FLIGHT8} DECISIONS[i] = 1;
   Flight9:
     sum {i in FLIGHT9} DECISIONS[i] = 1;
   Flight10:
     sum {i in FLIGHT10} DECISIONS[i] = 1;
   Flight11:
     sum {i in FLIGHT11} DECISIONS[i] = 1;
   Flight12:
     sum {i in FLIGHT12} DECISIONS[i] = 1;

